#!/bin/bash

if [ ${#} != 3 ]; then
	echo "ERROR: 3 parameters expected, ${#} found."
	echo "Usage: ${0} <tasksim_config_file> <output_trace> <trace>"
	if [ "`uname -m`" = "riscv64" ] ; then
		echo "Example ${0} /gpfs/scratch/bsc18/bsc18292/romol/musa_redistributable/tasksim/etc/conf/riscv_reference.conf ./output_trace /apps/paraver_sample_traces/fft/RISCV_fft.1024"
	else
		echo "Example ${0} /gpfs/scratch/bsc18/bsc18292/romol/musa_redistributable/tasksim/etc/conf/riscv_reference.conf ./output_trace /apps/TASKSIM/input_samples/apxy/axpy_4096_in128"
	fi
	exit 1;
elif [ ! -f "${1}" ]; then
	echo "ERROR: tasksim configuration file ${1} not found."
	exit 1;
# elif [ ! -d `echo "${2}" | rev | cut -d '/' -f1 | rev` ]; then
#	 echo "ERROR: Output directory does not exist. You need to create it."
#	 exit 1;
elif [ ! -f "${3}.prv" ]; then
	echo "ERROR: ${3}.prv file is missing from the trace files."
	exit 1;
elif [ ! -f "${3}.pcf" ]; then
	echo "ERROR: ${3}.pcf file is missing from the trace files."
	exit 1;
elif [ ! -f "${3}.row" ]; then
	echo "ERROR: ${3}.row file is missing from the trace files."
	exit 1;
fi



if [ -f "${2}".pcf ] || [ -f "${2}".prv ] || [ -f "${2}".row ] || [ -f "${2}".stats ]; then
	echo "WARNING: Output files ${2} will be overwritten."
fi

if [ ! -f `grep "latency_file" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: latency file is missing."
	latency_file=""
else
	latency_file=`grep "latency_file" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

if [ -z `grep "inorder_commit" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: inorder_commit flag not specified in the config file."
	inorder_commit=0
else
	inorder_commit=`grep "inorder_commit" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

if [ -z `grep "register_file_length" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: register_file_length not specified in the config file."
	vector_unit_throughput=512
else
	vector_unit_throughput=`grep "register_file_length" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

config_file=${1}
result=${2}
trace=${3}

ts=`echo ${0} | rev | cut --complement -d '/' -f1 | rev`
binary=riscvsim

echo "Checking configuration file parameters..."
${ts}/check_config_file -c ${config_file} -x ${ts}/../etc/conf/parameter_checklist.txt
if [ $? -ne 0 ]; then
    echo "Configuration file had some errors, correct them before running the simulation."
    exit
else
    echo "Configuration file has all necessary parameters."
fi
start=`date +%s`

echo "Simulating ${ts}/${binary} ${config_file} ${trace} -c MemCPU=RISCV -c MemCPU.prv_trace=${result} -c prv_trace=${result} -c memory_model=PERFECT -c private_cache_levels=1 -c shared_cache_levels=0 &> ${result}.stats"
${ts}/${binary} ${config_file} ${trace} -c MemCPU=RISCV -c MemCPU.prv_trace=${result} -c prv_trace=${result} -c memory_model=PERFECT -c private_cache_levels=1 -c shared_cache_levels=0 &> ${result}.stats
if [ $? -ne 0 ]; then
    end=`date +%s`
    secs=$(( (end - start) ))
    time=`date -d@${secs} -u +%H:%M:%S`
    echo "Simulation ended after ${time} with error code $?, printing simulator output ${result}.stats"
    cat ${result}.stats
else
    end=`date +%s`
    secs=$(( (end - start) ))
    time=`date -d@${secs} -u +%H:%M:%S`
    echo "Simulation done. It took ${time}."
fi

dpt=$(grep "disable_paraver_trace" ${config_file})
dpt=${dpt%%#*}
dpt=${dpt##*=}
if [[ ! -z $dpt ]]; then
    if [[ $dpt -eq 0 ]]; then
        cp ${trace}.row ${result}.row
    fi
else
    cp ${trace}.row ${result}.row
fi
